@@ -23,7 +23,7 @@ import io.fabric8.kubernetes.api.model.Pod;
 import io.fabric8.kubernetes.api.model.Secret;
 import io.fabric8.kubernetes.api.model.Service;
 import io.fabric8.kubernetes.api.model.ServiceAccount;
-import io.fabric8.kubernetes.api.model.extensions.Ingress;
+import io.fabric8.kubernetes.api.model.networking.v1.Ingress;
 import io.fabric8.kubernetes.client.KubernetesClient;
 import io.fabric8.kubernetes.client.KubernetesClientException;
 import io.fabric8.kubernetes.client.Watch;
@@ -132,13 +132,19 @@ public class KubernetesNamespace {
    * namespace already exists or we create new one. If update labels operation fail due to lack of
    * permission, we do not fail completely.
    *
+   * <p>The method will try to annotate the namespace with provided `annotations`. It does not
+   * matter if the namespace already exists or we create new one. If update annotations operation
+   * fail due to lack of permission, we do not fail completely.
+   *
    * @param canCreate defines what to do when the namespace is not found. The namespace is created
    *     when {@code true}, otherwise an exception is thrown.
    * @param labels labels that should be set to the namespace
+   * @param annotations annotations that should be set to the namespace
    * @throws InfrastructureException if any exception occurs during namespace preparation or if the
    *     namespace doesn't exist and {@code canCreate} is {@code false}.
    */
-  void prepare(boolean canCreate, Map<String, String> labels) throws InfrastructureException {
+  void prepare(boolean canCreate, Map<String, String> labels, Map<String, String> annotations)
+      throws InfrastructureException {
     KubernetesClient client = clientFactory.create(workspaceId);
     Namespace namespace = get(name, client);
 
@@ -150,6 +156,7 @@ public class KubernetesNamespace {
       namespace = create(name, client);
     }
     label(namespace, labels);
+    annotate(namespace, annotations);
   }
 
   /**
@@ -204,6 +211,58 @@ public class KubernetesNamespace {
     }
   }
 
+  /**
+   * Applies given `ensureAnnotations` into given `namespace` and update the `namespace` in the
+   * Kubernetes.
+   *
+   * <p>If we do not have permissions to do so (code=403), this method does not throw any exception.
+   *
+   * @param namespace namespace to annotate
+   * @param ensureAnnotations these annotations should be applied on given `namespace`
+   * @throws InfrastructureException if something goes wrong with update, except lack of permissions
+   */
+  protected void annotate(Namespace namespace, Map<String, String> ensureAnnotations)
+      throws InfrastructureException {
+    if (ensureAnnotations.isEmpty()) {
+      return;
+    }
+    Map<String, String> currentAnnotations = namespace.getMetadata().getAnnotations();
+    Map<String, String> newAnnotations =
+        currentAnnotations != null ? new HashMap<>(currentAnnotations) : new HashMap<>();
+
+    if (newAnnotations.entrySet().containsAll(ensureAnnotations.entrySet())) {
+      LOG.debug(
+          "Nothing to do, namespace [{}] already has all required annotations.",
+          namespace.getMetadata().getName());
+      return;
+    }
+
+    try {
+      // update the namespace with new annotations
+      cheSAClientFactory
+          .create()
+          .namespaces()
+          .createOrReplace(
+              new NamespaceBuilder(namespace)
+                  .editMetadata()
+                  .addToAnnotations(ensureAnnotations)
+                  .endMetadata()
+                  .build());
+    } catch (KubernetesClientException kce) {
+      if (kce.getCode() == 403) {
+        LOG.warn(
+            "Can't annotate the namespace due to lack of permissions. Grant cluster-wide permissions "
+                + "to `get` and `update` the `namespaces` to the `che` service account "
+                + "(Che operator might have already prepared a cluster role called "
+                + "`che-namespace-editor` for this, depending on its configuration). "
+                + "Alternatively, consider disabling the feature by setting "
+                + "`che.infra.kubernetes.namepsace.annotate` to `false`.");
+        return;
+      }
+      throw new InfrastructureException(kce);
+    }
+  }
+
   /**
    * Deletes the namespace. Deleting a non-existent namespace is not an error as is not an attempt
    * to delete a namespace that is already being deleted.
